%% INTERVAL HALVING METHOD %%
%% Developed by Dr. Rajib Kuamr Bhattacharjya %%
%% Department of Civil Engineering, IIT Guwahati %%
%% This method is used to obtain the minima of a minimization function
%% 'a' and 'b' are the upper bound and lower bound of search space 
%% 'e' is the permissible error
%% funcName is the name of the function 
%% x1 and x2 are the lower and upper bound of the search space after end of
%% the iteration 
%% k is the number of iteration required to obtain the optima
%% An example function 'func' has been provided
%% For obtaining the minima of the function just excecute the
%% intervalHalving
%% function using "[x1 x3 k]=intervalHalving(0,4,0.001,'func')" 
%% Where a=0,b=4, e=0.001 and funcName='func

function [x1,x2,k]=intervalHalving(a,b,e,funcName)
% a is lower bound
% b is upper bound
% funcName is the name of the objective function

L=abs(b-a);
xm = (a+b)/2;
fxm = feval(funcName,xm);
k=1;
while (L>=e) 
    L = b-a;
    x1 = a+L/4;
    x2 = b-L/4;
    fx1 = feval(funcName,x1);
    fx2 = feval(funcName,x2);
    k=k+2;
    if (fx1<fxm)
        b = xm;
        xm = x1; fxm = fx1;
    elseif (fx2<fxm)
        a = xm;
        xm = x2; fxm = fx2;
    else
        a=x1;b=x2;
    end
end
